@@ -23,6 +23,8 @@ use strict;
 use warnings;
 use autodie;
 
+use Text::Levenshtein qw(distance);
+
 use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Tags qw(tag);
@@ -44,19 +46,19 @@ my $dh_addons_manual
 my $compat_level = Lintian::Data->new('debhelper/compat-level',qr/=/);
 
 my $MISC_DEPENDS = Lintian::Relation->new('${misc:Depends}');
+my $NAMED_COMPAT_LEVELS = Lintian::Data->new('debhelper/named-compat-levels',
+    qr/\s*=>\s*/, \&_named_compat_levels);
 
 sub run {
     my (undef, undef, $info) = @_;
     my $droot = $info->index_resolved_path('debian/');
-    my $drules;
+    my ($drules, $dh_bd_version, $level, $using_named_compat);
 
     my $seencommand = '';
     my $needbuilddepends = '';
     my $needdhexecbuilddepends = '';
     my $needtomodifyscripts = '';
-    my $level;
     my $compat = 0;
-    my $usescdbs = '';
     my $seendhcleank = '';
     my %missingbdeps;
     my %missingbdeps_addons;
@@ -65,8 +67,7 @@ sub run {
     my $dhcompatvalue;
     my $inclcdbs = 0;
 
-    my $bdepends_noarch;
-    my $bdepends;
+    my ($bdepends_noarch, $bdepends, %build_systems);
     my $seen_dh = 0;
     my $seen_python_helper = 0;
     my $seen_python3_helper = 0;
@@ -87,20 +88,19 @@ sub run {
             $maybe_skipping--;
         }
 
+        next if /^\s*\#/;
         if (m/^\s+-?(dh_\S+)/) {
             my $dhcommand = $1;
+            $build_systems{'debhelper'} = 1
+              if not exists($build_systems{'dh'});
 
-            if ($dhcommand eq 'dh_pysupport') {
-                tag 'dh_pysupport-is-obsolete', "line $.";
-                $seen_python_helper = 1;
-            }
             if ($dhcommand eq 'dh_installmanpages') {
                 tag 'dh_installmanpages-is-obsolete', "line $.";
             }
             if ($dhcommand eq 'dh_python3') {
                 $seen_python3_helper = 1;
             }
-            if ($dhcommand =~ m,^dh_(?:pysupport$|python(?:2$|\$.*)),) {
+            if ($dhcommand =~ m,^dh_python(?:2$|\$.*),) {
                 $seen_python_helper = 1;
             }
 
@@ -128,12 +128,14 @@ sub run {
             $seencommand = 1;
             $needbuilddepends = 1;
         } elsif (m,^\s+dh\s+,) {
+            $build_systems{'dh'} = 1;
+            delete($build_systems{'debhelper'});
             $seen_dh = 1;
             $seencommand = 1;
             $needbuilddepends = 1;
             $needtomodifyscripts = 1;
-            while (m/\s--with(?:=|\s+)(\S+)/go) {
-                my $addon_list = $1;
+            while (m/\s--with(?:=|\s+)(['"]?)(\S+)\1/go) {
+                my $addon_list = $2;
                 for my $addon (split(m/,/o, $addon_list)) {
                     $addon =~ y,-,_,;
                     my $depends =$dh_addons_manual->value($addon)
@@ -141,7 +143,7 @@ sub run {
                     if (defined $depends) {
                         $missingbdeps_addons{$depends} = $addon;
                     }
-                    if ($addon =~ m,python(?:2|_central|_support)$,) {
+                    if ($addon eq 'python2') {
                         $seen_python_helper = 1;
                     } elsif ($addon eq 'python3') {
                         $seen_python3_helper = 1;
@@ -156,19 +158,17 @@ sub run {
                 $seen_python_helper = 1;
                 $seen_python3_helper = 1;
             }
-            if ($seen_python_helper == 0) {
-                $seen_python_helper = -1; # maybe; we'll check that later
-            }
-        } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,) {
+        } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,
+            or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
+            $build_systems{'cdbs-with-debhelper.mk'} = 1;
+            delete($build_systems{'cdbs-without-debhelper.mk'});
             $seencommand = 1;
             $needbuilddepends = 1;
             $needtomodifyscripts = 1;
             $inclcdbs = 1;
 
-         # CDBS sets DH_COMPAT but doesn't export it.  It does, however, create
-         # a debian/compat file if none was found; that logic is handled later.
+            # CDBS sets DH_COMPAT but doesn't export it.
             $dhcompatvalue = $cdbscompat;
-            $usescdbs = 1;
         } elsif (/^\s*export\s+DH_COMPAT\s*:?=\s*([^\s]+)/) {
             $level = $1;
         } elsif (/^\s*export\s+DH_COMPAT/) {
@@ -177,11 +177,49 @@ sub run {
             $dhcompatvalue = $1;
             # one can export and then set the value:
             $level = $1 if ($level);
-        } elsif (/^override_dh_/) {
+        } elsif (/^([^:]*override_dh_[^:]*):/) {
+            my $targets = $1;
             $needbuilddepends = 1;
-        } elsif (m,^include\s+/usr/share/cdbs/,
-            or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
+            # Can be multiple targets per rule.
+            while ($targets =~ /\boverride_(dh_[^\s]+)/g) {
+                my $dhcommand = $1;
+                # If maintainer is using wildcards, it's unlikely to be a typo.
+                next if ($dhcommand =~ /%/);
+                next if ($dh_commands_depends->known($dhcommand));
+                # Unknown command, so check for likely misspellings
+                foreach my $x (sort $dh_commands_depends->all) {
+                    if (distance($dhcommand, $x) < 3) {
+                        tag 'typo-in-debhelper-override-target',
+                          "override_$dhcommand", '->', "override_$x",
+                          "(line $.)";
+                        last; # Only emit a single match
+                    }
+                }
+            }
+        } elsif (m,^include\s+/usr/share/cdbs/,) {
             $inclcdbs = 1;
+            $build_systems{'cdbs-without-debhelper.mk'} = 1
+              if not exists($build_systems{'cdbs-with-debhelper.mk'});
+        } elsif (
+            m{
+              ^include \s+
+                 /usr/share/(?:
+                   dh-php/pkg-pecl\.mk
+                  |blends-dev/rules
+                 )
+              }xsm
+          ) {
+            # All of these indirectly use dh.
+            $build_systems{'dh'} = 1;
+            delete($build_systems{'debhelper'});
+        } elsif (
+            m{
+              ^include \s+
+                 /usr/share/pkg-kde-tools/qt-kde-team/\d+/debian-qt-kde\.mk
+              }xsm
+          ) {
+            $build_systems{'dhmk'} = 1;
+            delete($build_systems{'debhelper'});
         }
     }
     close($rules_fd);
@@ -192,6 +230,13 @@ sub run {
         tag 'unused-build-dependency-on-cdbs' if ($bdepends->implies('cdbs'));
     }
 
+    if (%build_systems) {
+        my @systems = sort(keys(%build_systems));
+        tag 'debian-build-system', join(', ', @systems);
+    } else {
+        tag 'debian-build-system', 'other';
+    }
+
     return unless $seencommand;
 
     my @pkgs = $info->binaries;
@@ -223,18 +268,27 @@ sub run {
         ($compat) = split(/\n/, $compat_file);
         strip($compat);
         if ($compat ne '') {
-            if ($compat !~ m/^\d+$/) {
+            my $compat_value = $compat;
+            my $named_compat = $NAMED_COMPAT_LEVELS->value($compat);
+            if (defined($named_compat)) {
+                $dh_bd_version = $named_compat->{'introduced-in'};
+                $compat_value = $named_compat->{'compat-level'};
+                $using_named_compat = 1;
+            } elsif ($compat !~ m/^\d+$/) {
                 tag 'debhelper-compat-not-a-number', $compat;
                 $compat =~ s/[^\d]//g;
+                $compat_value = $compat;
                 $compatnan = 1;
             }
             if ($level) {
+                my $c = $compat;
+                $c .= " ($compat_value)" if $using_named_compat;
                 tag 'declares-possibly-conflicting-debhelper-compat-versions',
-                  "rules=$level compat=$compat";
+                  "rules=$level compat=${c}";
             } else {
                 # this is not just to fill in the gap, but because debhelper
                 # prefers DH_COMPAT over debian/compat
-                $level = $compat;
+                $level = $compat_value;
             }
         } else {
             tag 'debhelper-compat-file-is-empty';
@@ -249,9 +303,6 @@ sub run {
         $compatnan = 1;
     }
 
-    if ($usescdbs and not defined($level)) {
-        $level = $cdbscompat;
-    }
     $level ||= 1;
     if ($level < $compat_level->value('deprecated')) {
         tag 'package-uses-deprecated-debhelper-compat-version', $level;
@@ -304,6 +355,16 @@ sub run {
             _tag_if_executable($file);
         } elsif ($basename =~ m/^ex\.|\.ex$/i) {
             tag 'dh-make-template-in-source', $file;
+        } elsif ($basename =~ m/^(?:(.*)\.)?maintscript$/) {
+            next unless $file->is_open_ok;
+            my $fd = $file->open;
+            while (<$fd>) {
+                if (m/--\s+"\$(?:@|{@})"\s*$/) {
+                    tag 'maintscript-includes-maint-script-parameters',
+                      $basename, "(line $.)";
+                }
+            }
+            close($fd);
         } elsif ($basename =~ m/^(?:.+\.)?debhelper(?:\.log)?$/){
             # The regex matches "debhelper", but debhelper/Dh_Lib does not
             # make those, so skip it.
@@ -380,18 +441,19 @@ sub run {
           unless ($bdepends_noarch->implies($dep));
     }
 
-    unless ($bdepends->implies("debhelper (>= $level~)")){
+    $dh_bd_version = $level if not defined($dh_bd_version);
+    unless ($bdepends->implies("debhelper (>= ${dh_bd_version}~)")){
         my $tagname = 'package-needs-versioned-debhelper-build-depends';
+        my @extra = ($level);
         $tagname = 'package-lacks-versioned-build-depends-on-debhelper'
-          if ($level <= $compat_level->value('pedantic'));
-
-        tag $tagname, $level;
+          if ($dh_bd_version <= $compat_level->value('pedantic'));
+        if ($using_named_compat) {
+            push(@extra, '(for the named compat level)');
+        }
+        tag $tagname, @extra;
     }
 
-    if ($seen_python_helper == -1 and $level >= 9) {
-        $seen_python_helper = 0;
-    }
-    if ($seen_dh and $seen_python_helper != 1) {
+    if ($seen_dh and not $seen_python_helper) {
         my %python_depends;
         for my $binpkg (@pkgs) {
             if ($info->binary_relation($binpkg, 'all')
@@ -400,17 +462,11 @@ sub run {
             }
         }
         if (%python_depends) {
-            if ($seen_python_helper == -1) {
-                $seen_python_helper
-                  = $bdepends_noarch->implies('python-support');
-            }
-            if (not $seen_python_helper) {
-                tag 'python-depends-but-no-python-helper',
-                  sort(keys %python_depends);
-            }
+            tag 'python-depends-but-no-python-helper',
+              sort(keys %python_depends);
         }
     }
-    if ($seen_dh and $seen_python3_helper != 1) {
+    if ($seen_dh and not $seen_python3_helper) {
         my %python3_depends;
         for my $binpkg (@pkgs) {
             if ($info->binary_relation($binpkg, 'all')
@@ -418,7 +474,7 @@ sub run {
                 $python3_depends{$binpkg} = 1;
             }
         }
-        if (%python3_depends and not $seen_python3_helper) {
+        if (%python3_depends) {
             tag 'python3-depends-but-no-python3-helper',
               sort(keys %python3_depends);
         }
@@ -488,7 +544,7 @@ sub _check_dh_exec {
                     $dhe_useless = 1;
                 }
             }
-            if ($dhe_useless) {
+            if ($dhe_useless && $path =~ /debian\/.*(install|manpages)/) {
                 my $form = $_;
                 chomp($form);
                 $form = "\"$form\"";
@@ -542,6 +598,16 @@ sub _shebang_cmd {
     return $cmd;
 }
 
+sub _named_compat_levels {
+    my ($key, $raw_val, undef) = @_;
+    my $result = {};
+    for my $opt (split(m/\s*,\s*/, $raw_val)) {
+        my ($key, $val) = split(m/\s*=\s*/, $opt, 2);
+        $result->{$key} = $val;
+    }
+    return $result;
+}
+
 1;
 
 # Local Variables:
